#!/usr/bin/env python

"""
usage: gen_django.py [-h] [-v] inschemaname outfilename

synopsis:
  generate django models from XML schema.

positional arguments:
  inschemaname   the schema file (file name or URL)
  outfilename    the name of the file to write the model to

optional arguments:
  -h, --help     show this help message and exit
  -v, --verbose  Print messages during actions.

examples:
  python gen_django.py myschema.xsd mymodel.py
notes:
  [to be added]

license:
  Copyright 2017 Dave Kuhlman

  Redistribution and use in source and binary forms, with or
  without modification, are permitted provided that the following
  conditions are met:

  1. Redistributions of source code must retain the above
  copyright notice, this list of conditions and the following
  disclaimer.

  2. Redistributions in binary form must reproduce the above
  copyright notice, this list of conditions and the following
  disclaimer in the documentation and/or other materials provided with
  the distribution.

  3. Neither the name of the copyright holder nor the names of its
  contributors may be used to endorse or promote products derived from
  this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""


#
# imports
from __future__ import print_function
import sys
import os
import argparse
import re
import schemalib


#
# Global variables

Opts = None
CleanPat = re.compile(r'[\-.:]')


#
# Classes

class Writer(object):
    """A convenient class to write output."""

    def __init__(self, outfile):
        self.outfile = outfile

    def write(self, *args):
        output = ' '.join(args)
        output = output + '\n'
        self.outfile.write(output)


#
# Private functions

def build_and_write_model(inschemaname, outfilename):
    if outfilename:
        outfile = make_file(outfilename)
    else:
        outfile = sys.stdout
    schema = schemalib.XsdSchemaClass()
    schema.parse(inschemaname)
    schema.build_schema()
    writer = Writer(outfile)
    wrt = writer.write
    write_model(schema, wrt)


def write_model(schema, wrt):
    wrt("from django.db import models")
    for complex_type in schema.complex_types:
        wrt("\n")
        wrt("class {}_model(models.Model):".format(
            clean_name(complex_type.name)))
        has_member = False
        for attribute in complex_type.attributes:
            if attribute.name != 'id':
                wrt("    {} = models.CharField(max_length=1000, )".format(
                    clean_name(attribute.name)))
        for member in complex_type.sequence:
            if member.type_name in schema.complex_type_map:
                wrt(
                    '    {0} = models.ForeignKey(\n'
                    '        "{1}_model",\n'
                    '        related_name="{2}_{0}_{1}",\n'
                    '    )'.format(
                        member.name,
                        member.type_name,
                        complex_type.name,
                    ))
            else:
                wrt("    {} = models.CharField(max_length=1000, )".format(
                    clean_name(member.name)))
            has_member = True
        wrt("")
        wrt(
            '    def __unicode__(self):\n'
            '        return "id: %s" % (self.id, )'
        )

        if not has_member:
            wrt("    pass")


def clean_name(name):
    new_name = re.sub(CleanPat, '_', name)
    return new_name


def make_file(outfilename):
    if os.path.exists(outfilename) and not Opts.force:
        sys.exit("outfilename exists.  Use --force to overwrite.")
    outfile = open(outfilename, 'w')
    return outfile


def dbg_msg(options, msg):
    """Print a message if verbose is on."""
    if options.verbose:
        print(msg)


#
# Exported functions


def main():
    global Opts
    description = """\
synopsis:
  generate django models from XML schema.
"""
    epilog = """\
examples:
  python gen_django.py myschema.xsd mymodel.py
notes:
  [to be added]
"""
    parser = argparse.ArgumentParser(
        description=description,
        epilog=epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "inschemaname",
        help="the schema file (file name or URL)"
    )
    parser.add_argument(
        "outfilename",
        help="the name of the file to write the model to"
    )
    parser.add_argument(
        "-f", "--force",
        action="store_true",
        help="force create and write output file without asking",
    )
    parser.add_argument(
        "-v", "--verbose",
        action="store_true",
        help="Print messages during actions.",
    )
    options = parser.parse_args()
    Opts = options
    schemalib.Opts = vars(options)
    inschemaname = Opts.inschemaname
    outfilename = Opts.outfilename
    build_and_write_model(inschemaname, outfilename)


if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    #import ipdb; ipdb.set_trace()
    main()
